/*

EXEMPLO.C
 
    aNDICE:
  
     1. Tratamento dos erros gerados pela biblioteca
     2. Deteccao automatica do ECF e conexao 
     3. Transferencia de dispositivos de memoria do ECF
         3.1. Rotina de callback da funcao SWEDA_CapturarDispositivo
         3.2. Funcao SWEDA_CapturarDispositivo
     4. Geracao do arquivo SINTEGRA  
         4.1. Informacoes do estabelecimento
         4.2. Rotina de callback da funcao SWEDA_CapturarDispositivo
     5. Reproducao da fita-detalhe para arquivo em codificacao ASCII
     6. Impressao da fita-detalhe
     7. Geracao do arquivo COTEPE
     8. Geracao da Tabela de Mercadorias e Servicos
     9. Geracao do arquivo SINTEGRA
    10. Reproducao da Leitura da Memoria Fiscal para arquivo em codificacao ASCII
    11. Impressao da Leitura da Memoria Fiscal
    12. Emissao do documento "Leitura X"
    13. Emissao do documento "Leitura de Parametros do ECF"
    14. Configuracao de parametros da biblioteca	  
        14.1. Leitura da configuracao atual
        14.2. Alteracao da configuracao atual
    15. Rotina auxiliar de exibicao de arquivos no console
    16. Modulo de ligacao dinamica da biblioteca
         16.1. Especificacao do nome do arquivo de ligacao dinamica
         16.2. Rotina de carregamento dinamico da biblioteca
         16.3. Rotina de descarregamento dinamico da biblioteca
    17. Menus da aplicacao
    18. Funcao main()
      18.1. Carregamento dinamico da biblioteca
      18.2. Execucao do menu principal	
      18.3. Descarregamento da biblioteca

*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "swmfd.h"      // Inclusao do arquivo header da biblioteca
#include "menus.cpp"    // Rotinas de gerenciamento de menus (console)


// 1. Tratamento de erros

short int Erro (short int Codigo)
{  
   if (Codigo < 0)
   {
      printf("\r\n processo interrompido pelo usuario\n");
      return 0;
   }
// Lista de mensagens referentes ao codigo de erro retornado por funcao da biblioteca
   const char *pszMensagemErro[] =
   {  "",
      "Falha de abertura do arquivo log!",
      "Data invalida!",
      "Faixa invalida!",
      "O numero de arquivos excede a capacidade da fila! (100 arquivos)",
 	  "Nenhum arquivo encontrado! (opcao: opArquivosPasta)",
	  "Nenhum arquivo selecionado! (opcao: opListaEmArquivo)",
	  "Arquivo NAO encontrado! (Origem)",
	  "Memoria insuficiente!",
	  "Falha em arquivo temporario!",
	  "NAO foi possvel abrir o arquivo de saida!",
	  "Arquivo invalido!",
	  "Falha na criacao da instancia!",
	  "O mes selecionado nao esta encerrado!",
	  "Buffer insuficiente",
      "Falha de gravacao em arquivo!",
      "Falha de comunicacao! (Timeout)",
      "Impressora fiscal esta' desligada ou NAO foi detectada!",
      "Tampa da impressora fiscal aberta!",
      "Impressora fiscal sem papel!",
      "Troque a bobina da impressora fiscal!",
      "Erro de sintaxe! Verifique os parametros da funcao.",
      "Comando invalido!",
      "Falha na execucao do comando fiscal!",
      "O aplicativo e' incompativel com o arquivo deste modelo de ECF ou com a versao de seu software basico!",
      "NAO houve movimento no periodo selecionado",
      "NAO e' possivel gerar o arquivo TDM a partir dos arquivos binarios informados. O "
      "movimento referente ao periodo selecionado NAO esta' registrado em ambos os "
      "dispositivos",
      "Arquivo NAO assinado",
      "Os arquivos de origem apresentam numeros de fabricacao distintos"
   };

   if (Codigo && Codigo < (short int) (sizeof (pszMensagemErro) / sizeof (*pszMensagemErro)))
       printf ("\r\n%s\n\n", pszMensagemErro [Codigo]);
   else
       if (Codigo != 0) 
          printf("\r\n ERRO DESCONHECIDO\n"); 

   return Codigo;
}


// 2. Deteccao automatica do ECF e conexao 

short int DetectarECF (struct MENU::ITEM *Item)
   
// Detecta automaticamente as configuracoes da porta de comunicacao serial 
// e obtem informacoes sobre o ECF conectado.

{  short int Porta;
   long Velocidade;
   struct TPropriedadesDispositivo PropriedadesECF;

   if (SWEDA_DetectarECF(0, &Porta, &Velocidade) == 0 ||
       SWEDA_ConectarECF (0, &PropriedadesECF, sizeof (PropriedadesECF)) == 0)
      printf ("\n\nERRO: ECF nao detectado!\n");
   else
      printf ("\n\n"
          "ECF detectado!\r\n\n"
          "   Modelo:     %s\r\n"
          "   FAB:        %s\r\n"
          "   MFD:        %s\r\n"
          "   Versao:     %s\r\n"
          "   Porta:      COM%d\r\n"
          "   Velocidade: %ld\r\n"
          , PropriedadesECF.ECF.Modelo
          , PropriedadesECF.ECF.FAB
          , PropriedadesECF.MFD.NS
          , PropriedadesECF.SB.Versao
          , Porta
          , Velocidade);

   return CONTINUAR;
}


// 3. Transferencia de dispositivos de memoria do ECF


// 3.1. Rotina de callback da funcao SWEDA_CapturarDispositivo

void CallBackCD (struct TInfoCallBackCD *Info, void *Parametro)

// Atraves da rotina de callback, a funcao SWEDA_CapturarDispositivo comunica a
// aplicacao eventos ocorridos durante a execucao.

{  switch (Info->Evento)
   {  case 200: printf ("Detectando o ECF...");
                break;
      case 110: printf ("\rVerificando dispositivos instalados...");
                break;
      case 130: printf ("\rArquivo: %-50s\r\n", Info->Arquivo);
                break;
      case 140: printf ("\rTransferindo... %ld de %ld bytes (%ld%%)", Info->Contador.Valor, Info->Contador.Maximo, (Info->Contador.Valor * 100) / Info->Contador.Maximo);
                break;
      case 198: printf ("\r\nProcesso interrompido pela aplicacao!\n");
                break;
      case 199: printf ("\r%-60s\n", "Transferencia concluida!");
                break;
   }
}


// 3.2. Funcao SWEDA_CapturarDispositivo

short int TransferirDispositivo (struct MENU::ITEM *Item)
{  printf ("\r\nDispositivo: %s\r\n", Item->Titulo);
   Erro (SWEDA_CapturarDispositivo ((short int) Item->Parametro, opAutoDetectarECF, 0, 0, 15000, CallBackCD, Item));
   return ENCERRAR_MENU;
}

/*

4. Geracao do arquivo SINTEGRA

4.1. Informaes do estabelecimento

Uma estrutura do tipo TDadosEstabelecimento e utilizada pela aplicacao para 
passar argumentos  funcao SWEDA_ArquivoSINTEGRA, que as utiliza para geracao 
dos registros tipo 10 e tipo 11 do arquivo SINTEGRA.

Opcionalmente, estas informaes podem ser configuradas na secao [SINTEGRA] 
do arquivo de inicilizacao da biblioteca, denominado SWMFD.INI:

[SINTEGRA]
   CNPJ: ""
   IE: ""
   IM: ""
   RAZaO SOCIAL: "SWEDA Informatica Ltda"
   LOGRADOURO: "Rua Dona Bragida"
   NUMERO: "713"
   COMPLEMENTO: ""
   BAIRRO: "Vila Mariana"
   CIDADE: "Sao Paulo"
   UF: "SP"
   CEP: "04111-081"
   TELEFONE: "11 5574-5644
   FAX: "5572-4799"
   CONTATO: "SAC"



*/
struct TDadosEstabelecimento Estabelecimento =
{  "",                           // CNPJ
   "",                           // Inscricao Estadual
   "",                           // Inscricao Municipal
   "Sweda Informatica Ltda",     // Nome comercial (Razao Social) do contribuinte
   "Rua Dona Brigida",           // Logradouro
   "713",                        // Numero
   "",                           // Complemento
   "Vila Mariana",               // Bairro
   "Sao Paulo",                  // Municpio onde esta domiciliado o estabelecimento informante
   "SP",                         // Unidade da Federacao referente ao Municpio
   "04111-081",                  // Codigo de Endereamento Postal
   "11 5574-5644",               // Numero de telefone para contato
   "5572-4799",                  // Fax
   "SAC"                         // Pessoa responsavel para contato
};


// 4.2. Rotina de callback da funcao SWEDA_CapturarDispositivo 

void CallBackSintegra (struct TInfoCallBackSintegra *Info, void *Parametro)

// Atraves da rotina de callback, a funcao SWEDA_CapturarDispositivo comunica 
// aplicacao eventos ocorridos durante a execucao.

{  switch (Info->Evento)
   {  case 110: printf ("Aguarde...");
             	break;
      case 200: printf ("\rDetectando o ECF...");
                break;
      case 130: printf ("\rTransferindo a memoria de fita-detalhe...\r\n");
                break;
      case 140: printf ("\r%ld/%ld bytes", Info->Contador.Valor, Info->Contador.Maximo);
                break;
      case 40:  printf ("\r\n\nCNPJ: %s\r\n", Info->Estabelecimento.CNPJ);

                if (*Info->Estabelecimento.IE)
                   printf ("IE: %s\r\n", Info->Estabelecimento.IE);

                if (*Info->Estabelecimento.IM)
                   printf ("IM: %s\r\n", Info->Estabelecimento.IM);

                if (*Info->Arquivo.FAB)
                   printf ("FAB: %s\r\n", Info->Arquivo.FAB);

                if (*Info->Arquivo.Loja)
                   printf ("Loja: %s\r\n", Info->Arquivo.Loja);

                if (*Info->Arquivo.ECF)
                   printf ("ECF: %s\r\n", Info->Arquivo.ECF);

                printf ("\r\n\n""Processando informacoes da memoria de fita-detalhe...\r\n");
                break;

      case 41:  printf ("\rCOO: %06ld/%06ld", Info->Contador.Valor, Info->Contador.Maximo);
                break;

      case 80:  printf ("\rConsolidando registros...\r\n");
                break;

      case 81:  printf ("\rRegistros: %ld/%ld", Info->Contador.Valor, Info->Contador.Maximo);
                break;
      case 98:  printf ("\rProcesso interrompido pela aplicacao!\n\n");
                break;
      case 99:  printf ("\rArquivo gerado com sucesso!\n\n");
                break;
   }
}


// Funcao auxiliar para exibicao de arquivos no console
void ExibirArquivo (char*);


short int SelecionarFaixa (struct MENU::ITEM *Item)
{  char Selecao[2][200];
   short int Opcao;
   char  soun;     
   const char *pszCriterio;
   char  pathwmf[512];
   char  pathwmfd[512];  

   Opcao = opTDM | opCodePage850;

   pszCriterio = (Item->Parametro & 32) ? "" : (Item->Parametro & 16 ? "o CRZ ou " : "o COO ou ");

   printf ("%s\r\n\n", Item->Titulo);
   printf ("Digite %sa data inicial (dd/mm/aaaa): ", pszCriterio);
   scanf ("%20s", Selecao[0]);
   printf ("Digite %sa data final: ", pszCriterio);
   scanf ("%20s", Selecao[1]);
   
   switch (Item->Parametro)
   {
// 5. Reproducao da fita-detalhe para arquivo em codificacao ASCII
      case 1:  if (Erro (SWEDA_ReproduzirFitaDetalhe ((char *) "",
                   (char *) "espelho-mfd.txt", Selecao[0], Selecao[1], opUTF_8, NULL, 0)))
                  return CONTINUAR;
               ExibirArquivo ((char *)  "espelho-mfd.txt");
               break;

// 6. Impressao da fita-detalhe
      case 2:  Erro (SWEDA_ImprimirFitaDetalhe (Selecao[0], Selecao[1]));
               return CONTINUAR;

// 7. Geracao do arquivo COTEPE
      case 20:  Opcao |= opCSV;
      case 19:  

        pathwmf[0] = pathwmfd[0] = '\0';  
        printf ("\nCaptura do ECF (s/n)?");
        do
          soun = getchar();
        while (toupper(soun) != 'S' && toupper(soun) != 'N');

        printf("\nAguarde....\n");
        if (soun == 'S' || soun == 's')
        {
          Erro (SWEDA_CapturarDispositivo (1, opAutoDetectarECF, pathwmf, sizeof(pathwmf),
          15000, CallBackCD, Item));

          Erro (SWEDA_CapturarDispositivo (2, opAutoDetectarECF, pathwmfd, sizeof(pathwmfd),
          15000, CallBackCD, Item));

          printf("MF  = %s\n", pathwmf);
          printf("MFD = %s\n", pathwmfd); 
        }
        printf("\n\nGerando COTEPE....\n");

        if (Erro (SWEDA_ArquivoEletronicoCOTEPE (pathwmf,pathwmfd,(char *) "COTEPE.TXT",
                                                Selecao[0], Selecao[1], Opcao)))
           return CONTINUAR;
        ExibirArquivo ((char *)  "COTEPE.TXT");
        break;

// 8. Geracao da Tabela de Mercadorias e Servios
      case 33: if (Erro (SWEDA_TabelaMercadoriasServicos ((char *) "TabelaProdutos.txt",
                            Selecao[0], Selecao[1], opCodePage850 | opOcultarJanela, 
                            (char *) "", (char *) "")))
                  return CONTINUAR;
               ExibirArquivo ((char *)  "TabelaProdutos.txt");
               break;

// 9. Geracao do arquivo SINTEGRA
      case 34: if (Erro (SWEDA_ArquivoSINTEGRA ((char *) "sintegra.txt", Selecao[0],
                        Selecao[1], &Estabelecimento,
		                  opSintegraTipo10 | opSintegraTipo11 | opSintegraTipo60M | opSintegraTipo60A | opSintegraTipo60D |
                  opSintegraTipo60I | opSintegraTipo60R | opSintegraTipo75 | opSintegraTipo90, opCodePage850 | opOcultarJanela,(char *) "", (char *) "LOG.TXT", CallBackSintegra, NULL, 0)))
               {  ExibirArquivo ((char *) "LOG.TXT");
                  return CONTINUAR;
               }
               ExibirArquivo ((char *)  "sintegra.txt");
               break;

// 10. Reproducao da Leitura da Memoria Fiscal para arquivo em codificacao ASCII
      case 17: if (Erro (SWEDA_ReproduzirLeituraMemoriaFiscal ((char *)"", 
                  (char *) "espelho-mf.txt", Selecao[0], Selecao[1], opLeituraCompleta, opUTF_8, NULL, 0)))
                  return CONTINUAR;
               ExibirArquivo ((char *)  "espelho-mf.txt");
               break;

// 11. Impressao da Leitura da Memoria Fiscal
      case 18: Erro (SWEDA_EmitirLeituraMemoriaFiscal (Selecao[0], Selecao[1], opLeituraCompleta));
               return ENCERRAR_MENU;
   }

   return CONTINUAR;
}

// 12. Emissao do documento "Leitura X"

short int EmitirLX (struct MENU::ITEM *Item)
{  Erro (SWEDA_EmitirLeituraX());
   return CONTINUAR;
}


// 13. Emissao do documento "Leitura de Parametros do ECF"

short int EmitirLP (struct MENU::ITEM *Item)
{  Erro (SWEDA_EmitirLeituraParametros());
   return CONTINUAR;
}



// 14. Configuracao de parametros da biblioteca	  

// Parametros de configuracao da biblioteca
struct SWMFD_INI Parametros;


extern struct MENU::ITEM MenuConfiguracao[];

short int OpcaoConfigurar (struct MENU::ITEM *Item)
{  
// 14.1. Leitura da configuracao atual
   SWMFD_ObterParametrosConfiguracao(&Parametros, sizeof (Parametros));

   MENU::Executar (MenuConfiguracao, Item->Titulo);
   Parametros.Estabelecimento = Estabelecimento;
	
// 14.2. Alteracao da configuracao atual
   SWMFD_AlterarParametrosConfiguracao(&Parametros, sizeof (Parametros));
	
   return CONTINUAR;
}


short int ConfigurarDiretorio (struct MENU::ITEM *Item)
{  return ENCERRAR_MENU;
}


short int ConfigurarPorta (struct MENU::ITEM *Item)
{  Parametros.Porta = (short int) Item->Parametro;
   printf ("\r\n\nOpcao configurada: %s\n", Item->Titulo);
   return ENCERRAR_MENU;
}


short int ConfigurarVelocidadeSerial (struct MENU::ITEM *Item)
{  Parametros.Velocidade = Item->Parametro;
   printf ("\r\n\nOpcao configurada: %s\n", Item->Titulo);
   return ENCERRAR_MENU;
}


// 15. Rotina auxiliar de exibicao de arquivos no console

void ExibirArquivo (char *Arquivo)
{  FILE *handle;
   short int Character, Linhas, Colunas;
   char szLinha[200];

   Colunas = Linhas = 0;
   handle = fopen (Arquivo, "rb");
   
   if (handle == 0)
	  return;
   
   printf ("\n");
            
   while (1)
   {  Character = fgetc (handle);
      if (Character == EOF)
      {  printf ("\n%-36s* Fim *\r\n", Arquivo);
         getchar();
         break;
      }

      if ((unsigned int) Colunas < (sizeof (szLinha) - 1))
         szLinha[Colunas++] = Character;

      if (Character == '\n')
      {  szLinha[Colunas] = '\0';
         printf ("%s", szLinha);
         Colunas = 0;
         if ((++Linhas % 23) == 0)
         {  printf ("%-30s> >  P A U S A  < <  (ENTRA para avancar ou ESC+ENTRA para sair)",
                    Arquivo);
            if (getchar() == 0x1B)
               break;
            printf ("\r%60s\r", "");
         }
      }
   }
   fclose (handle);
}


// 16. Modulo de ligacao dinamica da biblioteca

#include "swmfd_dl.c"


// 17. Menus da aplicacao


struct MENU::ITEM MenuSelecaoPorta [] =
{  { (char *)  "COM1", ConfigurarPorta, 1  },
   { (char *)  "COM2", ConfigurarPorta, 2  },
   { (char *)  "COM3", ConfigurarPorta, 3  },
   { (char *)  "COM4", ConfigurarPorta, 4  },
   { (char *)  "COM5", ConfigurarPorta, 5  },
   { (char *)  "COM6", ConfigurarPorta, 6  },
   { (char *)  "Detectar", ConfigurarPorta, 0  },
   { (char *)  "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuSelecaoVelocidade [] =
{  { (char *) "9.600 bps", ConfigurarVelocidadeSerial, 9600  },
   { (char *) "19.200 bps", ConfigurarVelocidadeSerial, 19200  },
   { (char *) "38.400 bps", ConfigurarVelocidadeSerial, 38400  },
   { (char *) "57.600 bps", ConfigurarVelocidadeSerial, 57600  },
   { (char *) "115.200 bps", ConfigurarVelocidadeSerial, 115200  },
   { (char *) "Detectar", ConfigurarVelocidadeSerial, 0  },
   { (char *) "Retornar"  },
   {  0  },
};

struct MENU::ITEM MenuComunicacao [] =
{  { (char *) "Porta", MENU::Submenu, (PARAMETRO) MenuSelecaoPorta  },
   { (char *) "Velocidade", MENU::Submenu, (PARAMETRO) MenuSelecaoVelocidade  },
   { (char *) "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuConfiguracao [] =
{  { (char *)  "Comunicacao", MENU::Submenu, (PARAMETRO) MenuComunicacao   },
//   { (char *)  "Diretorio de gravacao", ConfigurarDiretorio  },
   { (char *)  "Retornar"  },

   {  0  },
};


struct MENU::ITEM MenuTransferencia [] =
{  { (char *)  "Memoria Fiscal", TransferirDispositivo, opCapturarMF  },
   { (char *)  "Memoria de Fita-Detalhe", TransferirDispositivo, opCapturarMFD },
   { (char *)  "Software Basico", TransferirDispositivo, opCapturarSoftwareBasico  },
   { (char *)  "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuArquivosEletronicos [] =
{  { (char *)  "Reproducao da Fita-Detalhe", SelecionarFaixa, 1  },
   { (char *)  "Reproducao da Leitura de Memoria Fiscal", SelecionarFaixa, 17  },
   { (char *)  "Arquivo COTEPE", SelecionarFaixa, 19  },
   { (char *)  "Planilia Eletronica (.CSV)", SelecionarFaixa, 20  },
   { (char *)  "Arquivo SINTEGRA", SelecionarFaixa, 34  },
   { (char *)  "Tabela de Mercadorias e Servicos", SelecionarFaixa, 33  },
   { (char *)  "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuEmissao [] =
{  { (char *) "Leitura X", EmitirLX  },
   { (char *) "Leitura da Memoria Fiscal", SelecionarFaixa, 18  },
   { (char *) "Leitura de Parametros do ECF", EmitirLP  },
   { (char *) "Retornar"  },
   {  0  },
};


struct MENU::ITEM MenuPrincipal [] =
{  { (char *) "Transferencia de dispositivos", MENU::Submenu, (PARAMETRO) MenuTransferencia  },
   { (char *) "Geracao de arquivos eletronicos", MENU::Submenu, (PARAMETRO) MenuArquivosEletronicos  },
   { (char *) "Emissao de documentos", MENU::Submenu, (PARAMETRO) MenuEmissao  },
   { (char *) "Impressao da fita-detalhe", SelecionarFaixa, 2  },
   { (char *) "Deteccao automatica do ECF", DetectarECF  },
   { (char *) "Configuracao", OpcaoConfigurar  },
   { (char *) "Sair"  },
   {  0  },
};


// 18. Funcao main()

int main()
{  
// 18.1. Carregamento dinamico da biblioteca...
   SWMFD_LoadLibrary();
	
   printf ("\n\nSWMFD Versao: %d.%02d", SWEDA_SWMFDVersao() / 100, SWEDA_SWMFDVersao() % 100);

// 18.2. Execucao do menu principal	
   MENU::Executar (MenuPrincipal, (char *)  "MENU PRINCIPAL");
	
// 18.3. Descarregamento da biblioteca...
   SWMFD_FreeLibrary();
	
   return 0;
}
