#define __int64 long long
   
#ifndef SPED_REGISTROS_HPP
#define SPED_REGISTROS_HPP

#define REGISTRO_SPED        (1 << 14)
#define REGISTRO_C400        ((1 << 0) | REGISTRO_SPED)
#define REGISTRO_C405        ((1 << 1) | REGISTRO_SPED)
#define REGISTRO_C410        ((1 << 2) | REGISTRO_SPED)
#define REGISTRO_C420        ((1 << 3) | REGISTRO_SPED)
#define REGISTRO_C425        ((1 << 4) | REGISTRO_SPED)
#define REGISTRO_C460        ((1 << 5) | REGISTRO_SPED)
#define REGISTRO_C470        ((1 << 6) | REGISTRO_SPED)
#define REGISTRO_C490        ((1 << 7) | REGISTRO_SPED)
#define REGISTRO_C495        ((1 << 8) | REGISTRO_SPED)
                                          
#pragma pack(push, 1)


// REGISTRO C400: EQUIPAMENTO ECF
struct C400
{  char          REG[5];              // Texto fixo contendo "C400"
   char          COD_MOD[3];          // Codigo do modelo do documento fiscal, conforme a Tabela 4.1.1
   char          ECF_MOD[21];         // Modelo do equipamento 
   char          ECF_FAB[21];         // Numero de serie de fabricacao do ECF
   char          ECF_CX[4];           // Numero do caixa atribuido ao ECF
};


// REGISTRO C405: REDUCAO Z
struct C405
{  char          REG[5];              // Texto fixo contendo "C405"
   char          DT_DOC[9];           // Data do movimento a que se refere a Reducao Z
   long          CRO;                 // Posicao do Contador de Reinicio de Operacao
   long          CRZ;                 // Posicao do Contador de Reducao Z
   long          NUM_COO_FIN;         // Numero do Contador de Ordem de Operacao do ultimo documento emitido no dia. (Numero do COO na Reducao Z)
   __int64       GT_FIN;              // Valor do Grande Total final
   __int64       VL_BRT;              // Valor da venda bruta
};


// REGISTRO C410: PIS E COFINS TOTALIZADOS NO DIA
struct C410
{  char          REG[5];              // Texto fixo contendo "C410"
   __int64       VL_PIS;              // Valor total do PIS
   __int64       VL_COFINS;           // Valor total do COFINS
};


// REGISTRO C420: REGISTRO DOS TOTALIZADORES PARCIAIS DA REDUcao Z
struct C420
{  char          REG[5];              // Texto fixo contendo "C420"
   char          COD_TOT_PAR[8];      // Codigo do totalizador, conforme Tabela 4.4.6
   __int64       VLR_ACUM_TOT;        // Valor acumulado no totalizador, relativo a respectiva Reducao Z.
   short int     NR_TOT;              // Numero do totalizador quando ocorrer mais de uma situacao com a mesma carga tributaria efetiva.
   char          DESCR_NR_TOT[256];   // Descricao da situacao tributaria relativa ao totalizador parcial, quando houver mais de um com a mesma carga tributaria efetiva.
};

      
// REGISTRO C425: RESUMO DE ITENS DO MOVIMENTO DIaRIO
struct C425
{  char          REG[5];              // Texto fixo contendo "C425"
   char          COD_ITEM[15];        // Codigo do item (campo 02 do Registro 0200)
   long          QTD;                 // Quantidade acumulada do item
   char          UNID[7];             // Unidade do item (Campo 02 do registro 0190)
   __int64       VL_ITEM;             // Valor acumulado do item
   __int64       VL_PIS;              // Valor do PIS
   __int64       VL_COFINS;           // Valor da COFINS
};

// REGISTRO C460: DOCUMENTO FISCAL EMITIDO POR ECF
struct C460
{  char          REG[5];              // Texto fixo contendo "C460"
   char          COD_MOD[3];          // Codigo do modelo do documento fiscal, conforme a Tabela 4.1.1
   char          COD_SIT[3];          // Codigo da situacao do documento fiscal, conforme a Tabela 4.1.2
   long          NUM_DOC;             // Numero do documento fiscal (COO)
   char          DT_DOC[9];           // Data da emisso do documento fiscal
   __int64       VL_DOC;              // Valor total do documento fiscal
   __int64       VL_PIS;              // Valor do PIS
   __int64       VL_COFINS;           // Valor da COFINS
   char          CPF_CNPJ[15];        // CPF ou CNPJ do adquirente
   char          NOM_ADQ[61];         // Nome do adquirente
};


// REGISTRO C470: ITENS DO DOCUMENTO FISCAL EMITIDO POR ECF
struct C470
{  char          REG[5];              // Texto fixo contendo "C470"
   char          COD_ITEM[15];        // Codigo do item (campo 02 do Registro 0200)
   long          QTD;                 // Quantidade do item
   long          QTD_CANC;            // Quantidade cancelada, no caso de cancelamento parcial de item
   char          UNID[7];             // Unidade do item (Campo 02 do registro 0190)
   __int64       VL_ITEM;             // Valor total do item
   short int     CST_ICMS;            // Codigo da Situacao Tributaria, conforme a Tabela indicada no item 4.3.1.
   short int     CFOP;                // Codigo Fiscal de Operacao e Prestacao
   long          ALIQ_ICMS;           // Aliquota do ICMS - Carga tributaria efetiva em percentual
   __int64       VL_PIS;              // Valor do PIS
   __int64       VL_COFINS;           // Valor da COFINS
};


// REGISTRO C490: REGISTRO ANALiTICO DO MOVIMENTO DIaRIO
struct C490
{  char          REG[5];              // Texto fixo contendo "C490"
   short int     CST_ICMS;            // Codigo da Situacao Tributaria, conforme a Tabela indicada no item 4.3.1.
   short int     CFOP;                // Codigo Fiscal de Operacao e Prestacao
   long          ALIQ_ICMS;           // Aliquota do ICMS
   __int64       VL_OPR;              // Valor da operacao correspondente a combinacao DE CST_ICMS, CFOP, e aliquota do ICMS, incluidas as despesas acessorias e acrescimos
   __int64       VL_BC_ICMS;          // Valor acumulado da base de calculo do ICMS, referente a combinacao de CST_ICMS, CFOP, e aliquota do ICMS.
   __int64       VL_ICMS;             // Valor acumulado do ICMS, referente a combinacao de CST_ICMS, CFOP e aliquota do ICMS.
   char          COD_OBS[7];          // Codigo da observacao do lancamento fiscal (campo 02 do Registro 0460)
};


// REGISTRO C495: RESUMO MENSAL DE ITENS DO ECF POR ESTABELECIMENTO
struct C495
{  char          REG[5];              // Texto fixo contendo "C495"
   long          ALIQ_ICMS;           // Aliquota do ICMS
   char          COD_ITEM[15];        // Codigo do item (campo 02 do Registro 0200)
   long          QTD;                 // Quantidade acumulada do item
   long          QTD_CANC;            // Quantidade cancelada acumulada, no caso de cancelamento parcial de item
   char          UNID[7];             // Unidade do item (Campo 02 do registro 0190)
   __int64       VL_ITEM;             // Valor acumulado do item
   __int64       VL_DESC;             // Valor acumulado dos descontos
   __int64       VL_CANC;             // Valor acumulado dos cancelamentos
   __int64       VL_ACMO;             // Valor acumulado dos acrescimos
   __int64       VL_BC_ICMS;          // Valor acumulado da base de calculo do ICMS
   __int64       VL_ICMS;             // Valor acumulado do ICMS
   __int64       VL_ISEN;             // Valor das saidas isentas do ICMS
   __int64       VL_NT;               // Valor das saidas sob no-incidncia ou no tributadas pelo ICMS
   __int64       VL_ICMS_ST;          // Valor das saidas de mercadorias adquiridas com substituicao tributaria do ICMS
};


#pragma pack(pop)


#endif       // SPED_REGISTROS_HPP



