
#ifndef SWMFD_H
#define SWMFD_H

#define SWMFD_VERSAO 128

#ifndef __CPLUSPLUS__
#ifdef __cplusplus
#define __CPLUSPLUS__ __cplusplus
#endif
#endif

#include "registros.h"

#ifdef __CPLUSPLUS__
#define BOOL bool
#else
#define BOOL unsigned char
#endif

#define DLL_IMPORT
#define STDCALL

// Contantes para selecao de registros do SINTEGRA (funcoes SWEDA_SINTEGRA)
#define opSintegraTipo10         (1 << 0)
#define opSintegraTipo11         (1 << 1)
#define opSintegraTipo60M        (1 << 2)
#define opSintegraTipo60A        (1 << 3)
#define opSintegraTipo60D        (1 << 4)
#define opSintegraTipo60I        (1 << 5)
#define opSintegraTipo60R        (1 << 6)
#define opSintegraTipo60x        (opSintegraTipo60M | opSintegraTipo60A | opSintegraTipo60D | opSintegraTipo60I | opSintegraTipo60R)
#define opSintegraTipo75         (1 << 7)
#define opSintegraTipo90         (1 << 8)

// Constantes para selecao da performance da funcoes SWEDA_SINTEGRA
#define opArquivosPasta          (1 << 0)
#define opIncluirSubpastas       (opArquivosPasta | (1 << 1))
#define opListaEmArquivo         (1 << 2)
#define opTransferirMFD          (1 << 3)

// Constantes para selecao geral de performance de funcoes da biblioteca
#define opMostrarJanela           0
#define opAutoDetectarECF        (1 << 14)
#define opMinimizarJanela        (1 << 13)
#define opSegundoPlano           (1 << 12)
#define opOcultarJanela          (1 << 11)
#define opCancelamento           (1 << 10)
#define opAssinarArquivo         (1 << 9)

#define opWindowsCodePage        (0 << 7)
#define opISO8859_1              (0 << 7)
#define opCodePage860            (1 << 7)
#define opCodePage850            (2 << 7)
#define opUTF_8                  (3 << 7)
#define opUTF_16                 (4 << 7)
#define opUTF_32                 (5 << 7)

#define opFaixaCOO               (1 << 6) 

#define opLeituraSimplificada    0
#define opLeituraCompleta        1
#define opCOTEPE1704             0
#define opCSV                    2
#define opTDM                    4
#define opASCII                  8
#define opCriarArquivoTemporario 16
#define opRS2L                   32   // Razao Social na 2a. linha do cliche
#define opSPED                   32

#define opExcluirCOM1            (1 << 4)
#define opExcluirCOM2            (1 << 5)
#define opExcluirCOM3            (1 << 6)
#define opExcluirCOM4            (1 << 7)
#define opExcluirCOM5            (1 << 8)
#define opExcluirCOM6            (1 << 9)
#define opExcluirCOM7            (1 << 10)
#define opExcluirCOM8            (1 << 11)

#define opCapturarMFD             (1 << 0)
#define opCapturarMF              (1 << 1)
#define opCapturarSoftwareBasico  (1 << 2)


#define PONTEIRO_NULO         -300

#pragma pack (push, 1)

// Informacoes do contribuinte para geracao do arquivo SINTEGRA
struct TDadosEstabelecimento
{  char CNPJ[21],                       // C.N.P.J. (opcional)
        IE[21],                         // Inscricao Estadual (opcional)
        IM[21],                         // Inscricao Municipal (opcional)
        RazaoSocial[51],                // Nome comercial (Razao Social) do contribuinte
                                        // Endereco:
        Logradouro[51],                 //     - Logradouro
        Numero[11],                     //     - Numero
        Complemento[31],                //     - Complemento
        Bairro[41],                     //     - Bairro
        Municipio[41],                  //     - Municpio onde esta'domiciliado o estabelecimento informante
        UF[3],                          //     - Unidade da Federacao referente ao Municpio
        CEP[16],                        //     - CEP 
        Telefone[21],                   // Numero de telefone para contato
        FAX[21],                        // Fax
        Contato[41];                    // Pessoa responsavel para contato
        
};      

// Propriedades do arquivo binario retornadas pela funcao SWEDA_ExtrairInformacoesECF
struct TPropriedadesDispositivo
{
// Sobre o arquivo
   struct TArquivo
   {  short int Tipo;
      char Nome[260],
           Data[11],
           Horario[10],
           Dispositivo[40];
      long Extensao;
   } Arquivo;

// De identificacao do equipamento
   struct TECF
   {  char Loja[20],
           ECF[10],
           FAB[22],
           Marca[21],
           Modelo[21],
           Tipo[21],
           Berco;
   } ECF;

// Sobre usuario(s)
   struct TUsuarioECF
   {  char CNPJ[21],
           IE[21],
           IM[21],
           MFD[22],
           Moeda[7],
           GT[25];
   } Usuario[12];
   short int Usuarios;

// Sobre o software basico
   struct TArquivoSB
   {  char Versao[12];              
      char Checksum[9];
   } SB;

// Sobre a MFD
   struct TArquivoMFD
   {  char NS[22];
      char Berco;
      long Capacidade;
      long Consumo;
      char DataInstalacao[11];
   } MFD;

   struct TContadores
   {  char COO[7],
           CRZ[7],
           CRO[7];
   } Contadores;

// Sobre a MF
   struct TArquivoMF
   {  char Berco;
      long Capacidade;
      long Consumo;
      char DataInstalacao[11];
   } MF;
};


// Informacoes sobre o arquivo de origem (Sintegra)
struct TArquivoOrigem
{  char Nome[260];         // Arquivo de origem
   char Data[11];          // Data do arquivo
   char Horario[10];       // Horario do arquivo
   char *Dispositivo;      // Descricao do dispositivo
   char Loja[5];           // Loja
   char ECF[4];            // Numero sequencial do ECF
   char FAB[22];           // N/S de fabricacao do ECF
   short int Tipo;
   long Extensao;          // Extensao do arquivo em bytes
};

// Contador
struct TContador
{  long Minimo;            // Valor minimo do contador
   long Maximo;            // Valor maximo do contador
   long Valor;             // Valor corrente do contador
};


// Informacoes repassadas pela funcao SWEDA_ArquivoSINTEGRA na rotina de callback da aplicacao.

struct TInfoCallBackSintegra
{  struct TDadosEstabelecimento Estabelecimento; // Dados sobre o estabelecimento
   struct TArquivoOrigem Fila[100];  // Fila de arquivos para processamento
   short int Arquivos;               // Numero total de arquivos
   struct TArquivoOrigem Arquivo;    // Informacoes do arquivo em processamento
   short int Sequencial;             // Numero sequencial do arquivo
   struct TContador Contador;        // Contador
   long TempoEstimado;               // Tempo estimado de conclusao em caso de transferencia da MFD
   char Destino[260];                // Nome do arquivo destino
   short int Evento;                 // Mensagem
   short int Retorno;                // Codigo de retorno na sada da funcao
   short int Tipo;                   // Tipo do registro (Mensagem 50)
   char Subtipo;                     // Subtipo do registro (Mensagem 50)
   char *Registro;                   // Registro (Mensagem 50)
};


// Informacoes retornadas no callback do SPED.
struct TInfoCallBackExp
{  struct TArquivoOrigem      // Informacoes do arquivo em processamento
   {  char Nome[260];         // Arquivo de origem
      char Data[11];          // Data do arquivo
      char Horario[10];       // Horario do arquivo
      char *Dispositivo;      // Descricao do dispositivo
      char Loja[5];           // Loja
      char ECF[4];            // Numero sequencial do ECF
      char FAB[22];           // N/S de fabricacao do ECF
      short int Tipo;
      long Extensao;          // Extensao do arquivo em bytes
      char CNPJ[21];          // CNPJ
      char IE[21];            // IE
      char IM[21];            // IM
      char Modelo[21];
   } Arquivo;

   short int Formato;         // Formato de exportacao
   char Destino[260];         // Nome do arquivo destino
   TContador Contador;        // Contadores - Indicadores de progrecao
   short int Evento;          // Codigo do evento
   short int Retorno;         // Codigo retornado para a aplicacao
   void *Adicional;           // Informacao especfica, comunicada no evento   
   long TempoTransferencia;   // Tempo estimado de conclusao em caso de transferencia da MFD, em segundos
};

// Informacoes comunicadas ao apliccao durante a execucao da funcao SWEDA_CapturarDispositivo 
// atraves da rotina de callback.

struct TInfoCallBackCD
{  char Arquivo[260];         // Arquivo de origem
   char NS[22];               // N/S do dispositivo
   char FAB[22];              // N/S de fabrica��o do ECF
   char Modelo[22];           // N/S do dispositivo
   short int Evento;          // Evento
   short int Dispositivo;     // Descricao do dispositivo
   struct TContador Contador; // Contador
   short int Retorno;         // Codigo retornado para aplicacao
   long TempoEstimado;        // Tempo estimado para conclusao em segundos
   long TaxaRecepcao;         // Taxa media de recepcao
};

// PARAMETROS SPED
struct TParametrosSPED
{  short int Perfil;          // Perfil de registros do arquivo 'A', 'B' ou 'C'
   char COD_OBS[7];           // Codigo da observacao do lancamento fiscal 
   unsigned short int CFOP;   // Codigo Fiscal de Operacao e Prestacao
   unsigned short int AliquotaPIS;     // Alquota para calculo do PIS
   unsigned short int AliquotaCOFINS;  // Alquota para calculo da COFINS 
};

// Parametros configurados pelo arquivo SWMFD.INI
struct SWMFD_INI
{  char DiretorioSweda[260];
   char FAB[22];
   char MFD[22];
   short int Porta;
   long Velocidade;
   short int ControleFluxo;
   struct TDadosEstabelecimento Estabelecimento;
   struct TAssinaturaDigital
   {  short int Parametros[32];
      struct TChaveRSA
      {  short int Extensao;
         unsigned char Modulo[256];
         unsigned char Expoente[256];
      } ChavePublica[2];
      unsigned Reserva[1024];
   } AD;
   long Parametros[32];

   struct TParametrosSPED SPED;
};

#pragma pack(pop)

#ifdef __CPLUSPLUS__
extern "C" {
#endif
	
typedef void (STDCALL *TCallBackSintegra) (struct TInfoCallBackSintegra*, void *Instancia);
typedef void (STDCALL *TCallBackCD) (struct TInfoCallBackCD*, void *Instancia);
typedef void (STDCALL *TEventoConexaoPortaFisco) (struct TPropriedadesDispositivo*);
typedef void (STDCALL *TCallBackExp) (TInfoCallBackExp*, void *Instancia);

#ifdef CARREGAMENTO_DINAMICO

// Prototipos para carregamento dinamico da biblioteca

short int (STDCALL *SWEDA_SWMFDVersao) ();
short int (STDCALL *SWEDA_ReproduzirLeituraMemoriaFiscal) (char *szArquivoOrigem, char *szArquivoDestino, char *SelecaoInicial, char *SelecaoFinal, short int Tipo, short int Opcoes, char *Buffer, long Extensao);
short int (STDCALL *SWEDA_ReproduzirFitaDetalhe) (char *szArquivoOrigem, char *szArquivoDestino, char *SelecaoInicial, char *SelecaoFinal, short int Opcoes, char *Buffer, long Extensao);
short int (STDCALL *SWEDA_ArquivoEletronicoCOTEPE) (char *pszOrigem1, char *pszOrigem2, char *pszArquivoDestino, char *pszSelecaoInicial, char *pszSelecaoFinal , short int Opcoes);
short int (STDCALL *SWEDA_DetectarECF) (short int Opcoes, short int *PortaDetectada, long *VelocidadeDetectada);
short int (STDCALL *SWEDA_TabelaMercadoriasServicos) (char *pszArquivoDestino, char *pszSelecaoInicial, char *pszSelecaoFinal, short int Opcoes, char *pszOrigem, char *pszArquivoLog);

short int (STDCALL *SWEDA_ExtrairInformacoesECF) (char *szNomeArquivo, struct TPropriedadesDispositivo *Buffer, short int Extensao, short int);
short int (STDCALL *SWEDA_CapturarDispositivo) (short int Dispositivo, short int Opcoes, char *Arquivo, short int Extensao, long Timeout, TCallBackCD CallBack, void *Instancia);
short int (STDCALL *SWEDA_ArquivoSINTEGRA) (char *, char *, char *, struct TDadosEstabelecimento*, short int, short int, char*, char*, TCallBackSintegra, void*, short int);
short int (STDCALL *SWEDA_EmitirLeituraX) ();
short int (STDCALL *SWEDA_EmitirLeituraMemoriaFiscal) (char*, char*, BOOL);
short int (STDCALL *SWEDA_EmitirLeituraParametros) ();
short int (STDCALL *SWEDA_ImprimirFitaDetalhe) (char*, char*);

     BOOL (STDCALL *SWEDA_ConectarECF) (short int, struct TPropriedadesDispositivo*, short int);

     void (STDCALL *SWMFD_ObterParametrosConfiguracao) (struct SWMFD_INI*, short int);
     void (STDCALL *SWMFD_AlterarParametrosConfiguracao) (struct SWMFD_INI*, short int);

     BOOL (STDCALL *SWMFD_AbrirPortaFiscal) (short int, TEventoConexaoPortaFisco);
     void (STDCALL *SWMFD_FecharPortaFiscal) ();
     short int (STDCALL *SWEDA_ArquivoSPED) (char*, char*, char*, char*, TParametrosSPED*, short int, short int, TCallBackExp, void*); 	 

#else
	 
// Ligacao dinamica

DLL_IMPORT short int STDCALL SWEDA_SWMFDVersao();
DLL_IMPORT short int STDCALL SWEDA_ExtrairInformacoesECF (char *szNomeArquivo, struct TPropriedadesDispositivo *Buffer, short int Extensao, short int);
DLL_IMPORT short int STDCALL SWEDA_ArquivoSINTEGRA (char *, char *, char *, struct TDadosEstabelecimento*, short int, short int, char*, char*, TCallBackSintegra, void*, short int);
DLL_IMPORT short int STDCALL SWEDA_ReproduzirLeituraMemoriaFiscal (char *szArquivoOrigem, char *szArquivoDestino, char *SelecaoInicial, char *SelecaoFinal, short int Tipo, short int Opcoes, char *Buffer, long Extensao);
DLL_IMPORT short int STDCALL SWEDA_ReproduzirFitaDetalhe (char *szArquivoOrigem, char *szArquivoDestino, char *SelecaoInicial, char *SelecaoFinal, short int Opcoes, char *Buffer, long Extensao);
DLL_IMPORT short int STDCALL SWEDA_ArquivoEletronicoCOTEPE (char *pszOrigem1, char *pszOrigem2, char *pszArquivoDestino, char *pszSelecaoInicial, char *pszSelecaoFinal, short int Opcoes);
DLL_IMPORT short int STDCALL SWEDA_CapturarDispositivo (short int Dispositivo, short int Opcoes, char *Arquivo, short int Extensao, long Timeout, TCallBackCD CallBack, void *Instancia);
DLL_IMPORT short int STDCALL SWEDA_DetectarECF (short int Opcoes, short int *PortaDetectada, long *VelocidadeDetectada);
DLL_IMPORT short int STDCALL SWEDA_TabelaMercadoriasServicos (char *pszArquivoDestino, char *pszSelecaoInicial, char *pszSelecaoFinal, short int Opcoes, char *pszOrigem, char *pszArquivoLog);

DLL_IMPORT short int STDCALL SWEDA_EmitirLeituraX ();
DLL_IMPORT short int STDCALL SWEDA_EmitirLeituraMemoriaFiscal (char*, char*, BOOL);
DLL_IMPORT short int STDCALL SWEDA_EmitirLeituraParametros();
DLL_IMPORT short int STDCALL SWEDA_ImprimirFitaDetalhe (char*, char*);
//DLL_IMPORT      BOOL STDCALL SWEDA_ConectarECF(short int, struct TPropriedadesDispositivo*, short int);

DLL_IMPORT      void STDCALL SWMFD_ObterParametrosConfiguracao (struct SWMFD_INI*, short int);
DLL_IMPORT      void STDCALL SWMFD_AlterarParametrosConfiguracao (struct SWMFD_INI*, short int);

DLL_IMPORT      BOOL STDCALL SWMFD_AbrirPortaFiscal (short int, TEventoConexaoPortaFisco);
DLL_IMPORT      void STDCALL SWMFD_FecharPortaFiscal ();
DLL_IMPORT short int STDCALL SWEDA_ArquivoSPED (char*, char*, char*, char*, TParametrosSPED*, short int, short int, TCallBackExp, void*); 	 

#endif

#ifdef __CPLUSPLUS__
};
#endif


#endif
